#!/usr/bin/perl
use strict;
use warnings;
use Bio::EnsEMBL::Registry;
use Bio::EnsEMBL::ApiVersion;

#http://asia.ensembl.org/info/docs/api/core/core_tutorial.html
#http://may2012.archive.ensembl.org/Homo_sapiens/Gene/Summary?g=ENSG00000185960;r=X:585079-620146

printf( "The API version used is %s\n", software_version() );


my $registry = 'Bio::EnsEMBL::Registry';

$registry->load_registry_from_db(
  -host => 'ensembldb.ensembl.org',
  -user => 'anonymous'
);

my @db_adaptors = @{ $registry->get_all_DBAdaptors() };   

id2transcripts("Human", "ENSG00000185960", 0);
#id2transcripts("Cave fish", "ENSAMXG00000015826", 0);

####
sub id2transcripts {
  my($spNameSR, $geneID, $extension) = @_;  
  
  #$spNameSR =~ s/ //g;

  my $slice_adaptor = $registry->get_adaptor($spNameSR, 'Core', 'Slice' );
  my $slice         = $slice_adaptor->fetch_by_gene_stable_id($geneID, $extension);

  my $sliceMyGene  = $slice_adaptor->fetch_by_gene_stable_id($geneID);
  my @myGene = @{ $sliceMyGene->get_all_Genes() };
  my $myGene = shift@myGene;
  #my $strand_myGene = $myGene->strand();

  my $transcripts = $myGene->get_all_Transcripts();

  print "\n$geneID\n\n";
  while ( my $transcript = shift @{$transcripts} ) {
      my $tstring = feature2string($spNameSR, $transcript);
      print "$tstring\n";

      ## The spliced_seq() method returns the concatenation of the exon
      ## sequences. This is the cDNA of the transcript
      #print "cDNA: ", $transcript->spliced_seq(), "\n";
      
      # The translateable_seq() method returns only the CDS of the transcript
      print "CDS: ", $transcript->translateable_seq(), "\n";

      # The protein sequence is obtained from the translate() method. If the
      # transcript is non-coding, undef is returned.
      my $protein = $transcript->translate();
      
      print "Translation: ", ( defined $protein ? $protein->seq() : "None" ), "\n";
      print "\n";
  }
}
    

sub feature2string
{
    my ($spNameSR, $featureSR) = @_;

    my $stable_id  = $featureSR->stable_id();
    my $seq_region = $featureSR->slice->seq_region_name();
    my $start      = $featureSR->start();
    my $end        = $featureSR->end();
    my $strand     = $featureSR->strand();

    my $transcript_adaptor =
      $registry->get_adaptor( $spNameSR, 'Core', 'Transcript' );
    my $transcript = $transcript_adaptor->fetch_by_stable_id($stable_id);
    my $proteinID  = $transcript->translation()->stable_id();
    my $proteinLength = length($transcript->translate()->seq());

    return sprintf( "%s_%s: %s:%d-%d (%+d), %d residues",
        $proteinID, $stable_id, $seq_region, $start, $end, $strand, $proteinLength);
}